#!/bin/sh

# bb-network.sh
# BIG BROTHER NETWORK MONITORING SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.06b
# Jan 21, 1998
#
# This program is Copyright (c) 1997, 1998
# The MacLawran Group Inc.
# All Rights Reserved
#
# SEND THE TEST RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME IN bb-net"
if test ! "$LYNX"			# GET DEFINITIONS IF NEEDED
then
	. $BBHOME/etc/bbdef.sh		# INCLUDE STANDARD DEFINITIONS
fi

#
# SMM - NEW FOR 1.05
# ALLOW TESTING OF BANKS OF CONNECTIONS ONLY
# FORMAT OF THE LINE IS
# dialup name IP-ADDRESS COUNT
# dialup modem-bank1 132.220.50.1 16
#
$GREP "^dialup" $BBHOSTS |
while read line
do
	set $line
	if test "$#" != "4"
	then
		echo "MISSING ARGUMENTS FOR dialup"
		echo "Line format is:"
		echo "dialup pool-name start-ip count"
		exit 1
	fi
	NAME="$2"
	TTL="$4"
	set `echo $3 | $SED "s/\./ /g"`
	BASE="$1.$2.$3"
	START="$4"
	END=`expr $START + $TTL - 1`
	# echo "NAME IS $NAME"
	# echo "TTL IS $TTL"
	# echo "BASE IS $BASE"
	# echo "START IS $START"
	# echo "END IS $END"
	LINE="FROM $BASE.$START TO $END DATA "
	while test "$START" -le "$END"
	do
		TEST="${BASE}.$START"
		# echo $TEST
		$PING $PINGPAR1 $TEST $PINGPAR2 > $BBTMP/BBPING 2>&1
		$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
		if test $? != "0"
		then
			LINE="$LINE clear "
		else
			LINE="$LINE green "
		fi
		START=`expr $START + 1`
	done
	# echo "LINE IS $LINE"
	$BB $BBDISP "status dialup.${NAME} green `date` $LINE" 
done

$GREP "^[1-9]" $BBHOSTS |
while read line
do
	set $line
	HOSTIP=$1			# HOST IP ADDRESS
	IPADDR=`echo $1 | $SED "s/\.//g"`
	FULLNAME=$2
	#
	# SMM - NEW FOR 1.04
	# USE FULLY QUALIFIED MACHINE NAMES?
	# DOT -> COMMA -> DOT FOR WEB DISPLAY
	#
	if test "$FQDN" = "TRUE"
	then
		set `echo $FULLNAME | $SED "s/\./,/g"`
	else
		set `echo $FULLNAME | $SED "s/\./ /"`
	fi
	MACHINE=$1
	#
	#================================================
	# CONNECTIVITY TEST
	# Added support for non-standard pings. gunnar@bitcon.no
	#
	# SMM: PING BY IP ADDRESS INSTEAD OF BY NAME. (v1.04a)
	# SMM: TRY THREE TIMES BEFORE PAGING (v1.06b)
	# 	BB SHOULD NOT BE A 3AM WAKE UP CALL...
	#================================================
	#
	COLOR="green"			# INNOCENT UNTIL PROVEN GUILTY...
	LINE="Connection OK"
	$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
	$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
	if test $? != "0"
	then
		sleep 20		# PAUSE... AND TRY AGAIN...
		$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
		$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
		if test $? != "0"
		then
			echo "$line" | $GREP "dialup" > /dev/null 2>&1
			if test $? = "0"
			then
				COLOR="clear"
				LINE="bb-network.sh: $FULLNAME offline..."
			else
				COLOR="red"
				LINE="bb-network.sh: Can't ping $FULLNAME..."
				sleep 30
			fi
			$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
			$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
			if test $? != "0"	# THREE STRIKES...
			then
				if test "$TELNETPAGE" = "Y"
				then
					$BB $BBPAGE "page ${TELNETERR}${IPADDR} ${LINE}"
					LINE="$LINE (paging)"
				fi
			else	# OK ON THIRD TRY... CAME BACK...
				COLOR="yellow"	
				LINE="Connection OK on 3rd try..."
			fi
		fi
	fi
	$BB $BBDISP "status ${MACHINE}.conn $COLOR `date` $LINE" 
	$RM -f $BBTMP/BBPING		# GET RID OF THE LEFTOVERS

	#
	# THIS IS NEW WITH 1.04... TEST FOR OTHER SERVICES HERE...
	# SEND THE RESULTS TO THE USUAL PLACE...
	#
	set $line			# GET ALL THE LINE ARGS
	shift; shift;			# SKIP THE IP-ADDR AND NAME

	while test "$1"			# AS LONG AS WE HAVE ARGS
	do
		line="$*"		# SAVE THIS..

		case $1 
		in

		nntp* | ftp* | pop* | smtp* )	# SERVICES WE KNOW ABOUT...
			#
			# OK... GET THE SERVICE NUMBER OUT OF /etc/services
			# CAN BE SEPERATED BY SPACES/TABS... (v1.04a)
			#
			SVCNAME=$1
			SVC=""
			set `$GREP "^${1}[ 	]" /etc/services | $GREP "/tcp"`
			while test "$SVC" = ""
			do
				echo $1 | $GREP "/tcp" > /dev/null
				if test "$?" = "0"
				then
					SVC=`echo $1 | $SED "s/\/tcp//"`
					#echo "FOUND SERVICE: $SVCNAME: $SVC"
				fi
				shift;
			done
			# echo "TESTING SERVICE $SVC"
			OUTPUT=`$BBHOME/bin/bbnet ${FULLNAME}:${SVC}` 
			if test "$?" != "0"
			then
				# echo "SERVICE IS BROKEN"
				#
				# IF IT'S BROKEN, THEN SCREAM IF WE HAVE TO
				#
                		LINE="SERVER - $SVCNAME DOWN"
                		echo "$PAGEPROC"|$GREP "$SVCNAME">/dev/null 2>&1
                		if test "$?" = "0"
                		then
                        		if test "$PAGER" != ""
                        		then
						# FIX BUG:MACHIP -> IPADDR
						# Bob Teeter <bobt@swdata.com>
		      				$BB $BBPAGE "page ${NETERR}${SVC}${IPADDR} ${MACHINE} $SVCNAME DOWN"
                                		LINE="${LINE} (paging)"
                        		fi
				fi
                        	COLOR="red"
			else
				LINE="$SVCNAME OK: $OUTPUT"
				COLOR="green"
				# echo $OUTPUT
			fi
			$BB $BBDISP "status ${MACHINE}.${SVCNAME} $COLOR `date` $LINE" 
			;;

		http* )	# echo "DOING HTTP TEST..."
			HTTP=$1
			$LYNX $HTTP > $BBTMP/HTTP 2>&1
			if test "$?" != 0
			then
				LINE="bb-network.sh: Lost HTTP connection to $HTTP"
				if test "$HTTPPAGE" != ""
				then
					$BB $BBPAGE "page ${HTTPERR}${IPADDR} $LINE"
					LINE="$LINE - Paging..."
				fi
				$BB $BBDISP "status ${MACHINE}.http red `date` $LINE"
			else
			#
			# CHECK FOR A SERVER ERROR HIDING HERE... CHECK FOR IT
			#

			#
			# THANKS TO: Douwe Dijkstra <D.Dijkstra@Twinfo.nl>
			# Version 1.04b
			#
				read HTTPVER RC STATUSMESSAGE < $BBTMP/HTTP
				case $RC in
    				'200'|'401'|'302')
         				LINE="Server is Ok : $RC $STATUSMESSAGE"
         				COLOR="green"
         				;;
    				*)
         				LINE="Server **NOT** Ok : $RC $STATUSMESSAGE"
         				COLOR="yellow"
         				;;
				esac
				$BB $BBDISP "status ${MACHINE}.http $COLOR `date` $LINE"
				$RM -f $BBTMP/HTTP
				HTTP=""
			fi
			;;
		esac
		set $line		# RESTORE...
		shift;			# GET NEXT ARGUMENT
	done
done
