/*
 * bbd.c 
 * BIG BROTHER DAEMON PROGRAM
 * Sean MacGuire - The MacLawran Group Inc.
 * Version 1.06b
 * Mar 15, 1998
 *
 * This program is Copyright (c) 1997, 1998
 * The MacLawran Group Inc.
 * All Rights Reserved
 *
 * This daemon just listens on a port, accepts single lines of input
 * and handles them.  It doesn't send any acknowledgements back...
 *
 * Two types of actions are possible, depending on the content we get:
 *
 *	1.	Input for the display monitor
 *	2.	Request to send a page to the admin
 */


#include "bb.h"		/* THE BIG BROTHER INCLUDE FILE */
#include <errno.h>

unsigned long securedb[MAXBUF];	/* SECURITY DB LIVES HERE */

char bblogdir[MAXLINE];		/* DIRECTORY TO PUT LOG FILES INTO */
char security[MAXLINE];		/* WHERE THE BBD SECURITY FILE LIVES */
char pager[MAXLINE];		/* BB-PAGER PROGRAM LOCATION */

main(argc, argv)
int argc;
char *argv[];
{
 	int status, sockfd, newsockfd, child, port, len;
	void cleanup();
        struct sockaddr_in serv_addr, cli_addr;
	static char line[MAXLINE];
	char *bbhome;				/* WHERE ALL BB LIVES */
	int on=1;
	unsigned long inc;

	debug("*** Starting bbd ***\n");
	bbhome=(char *)getenv("BBHOME");
	if (bbhome) {
		len=strlen(bbhome);
		if (len > MAXLINE - 50) {		/* CHECK THIS... */
			fprintf(stderr,"bbd: BBHOME is too large - go away!\n");
			exit(1);
		}
        	sprintf(security, "%s/etc/security", bbhome);
		sprintf(bblogdir, "%s/www/logs", bbhome);
		sprintf(pager, "%s/bin/bb-page.sh", bbhome);
	}
	else {
		fprintf(stderr,"bbd: BBHOME is not set in the environment!\n");
		fprintf(stderr,"Make sure you're running bbd from runbb.sh!\n");
		exit(1);
	}
	/*
	 * BB SECURITY ENHANCEMENT
	 * ALLOW BB TO RESTRICT ACCESS BASED ON A LIST OF 
	 * NUMERIC HOSTS/MASKS LOCATED IN etc/security
	 */
	getsecurity();			/* GET VALID HOSTS AND MASKS */

	port = PORT;

        debug("MAKING SOCKET CONNECTION...\n");
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "bbd: Can't open socket\n");
		exit(1);
        }
        debug("SOCKET CONNECTED OK...\n");

	/*
	 * FINALLY FIX THE REUSE BUGLET
	 * THANKS TO Gary Mills <mills@cc.UManitoba.CA>
	 */
	(void)setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof on);

        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
        serv_addr.sin_port = htons(port);

        debug("BINDING TO SOCKET...\n");
        if (bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr))<0) {
                fprintf(stderr,"bbd: Can't bind to port 1984 - ");
		fprintf(stderr, "is bbd already running?\n");
		exit(2);
        }
        debug("SOCKET BOUND OK...\n");

        /*
	 * NOW THE PARENT EXITS AND THE CHILD DETACHES FROM THE CONTROLLING TTY
	 * SET UP TO IGNORE SIGNALS SENT BY EXITING CHILDREN
         */
        if ((child = fork()) != 0) exit(0);             /* PARENT EXITS */
	setsid();					/* RELEASE TTY */

#ifndef ZOMBIE
        signal(SIGCHLD, SIG_IGN);
#endif /* NO ZOMBIE */

	/*
	 * NOW SET UP THE LISTENING LOOP
         */
        listen(sockfd, 5);

        for ( ;; ) {
                len = sizeof(cli_addr);
                newsockfd = accept(sockfd,(struct sockaddr*)&cli_addr,&len);
  		/*
                 * SMM BBD SECURITY CHECK
                 */
                inc = cli_addr.sin_addr.s_addr;
                debug("INCOMING CONNECTION FROM: %u\n", inc);
		if (checksecurity(inc) == 0) { /* FAIL */
                	debug("SECURITY: %s\n", inet_ntoa(inc));
			close(newsockfd);
			continue;
		}

		/*
		 * THANKS TO Marc L. Allen (allen@chesapeake.rps.slb.com)
		 */

                if (newsockfd < 0 ) {
                        if (errno == EINTR) continue;
			else {
				/* 
				 * Jac Kersing *kersing@the-box.com)
				 * Just retry. Might be a killed client 
				 */
				sleep(5);
				continue;
			}
		}

		/*
		 * NOW FORK OFF AND LET THE CHILD HANDLE THE REQUEST
		 */
 		if ((child = fork()) < 0) die("fork error");

		if (child != 0) {			/* THE PARENT */
			close(newsockfd);
#ifdef ZOMBIE
			debug("WAIT FOR CHILD EXIT...\n");
			wait(&status);
			debug("CLEANED UP CHILD: %d\n", status);
#endif /* ZOMBIE */
		}
		else {					/* THE CHILD */
			close(sockfd); 
			/*
			 * NOW GET THE LINE FROM THE CLIENT
			 * AND HANDLE THE REQUEST...
			 * IGNORE BAD DATA...
			 */
			len = recv(newsockfd, line, MAXLINE - 1, 0);
			line[MAXLINE]='\0';		/* FOR ANDREY */
			if (len > 0) do_bb(line);
			close(newsockfd);
                	exit(0);
		}
	}
}

/*
 * HANDLE INCOMING BIG BROTHER REQUESTS
 * PARSE THE LINE WE'VE RECEIVED AND HANDLE IT
 */
do_bb(req)
char *req;
{
	debug("REC'D: %s\n", req);
	/*
	 * IF THE REQUEST STARTS WITH "page" THEN CALL THE PAGING SCRIPT
	 * IF THE REQUEST STARTS WITH "status" THEN LOG THE MESSAGE
	 * OTHERWISE IGNORE IT (FOR SAFETY SAKE)
	 */
	if (strncmp(req, "page ", 5) == 0) {
		req += 5;			/* JUMP OVER page */
		debug("CALL PAGER: %s\n", req);
		if (pager) {
			execlp(pager, pager, req, (char *)NULL);
			die("SERVER PAGE GAGGED\n");
		}
		else die("PAGE FAILED!\n");
	}
	else if (strncmp(req, "status ", 7) == 0) {
		req += 7;			/* JUMP OVER status */
		debug("LOG STATUS MSG: %s\n", req);
		if (req) bblog(req);		/* LOG THE REQUEST */
		else debug("NULL STATUS MSG IGNORED\n");
	}
	else debug("IGNORE MESSAGE: %s\n", req);
}

/*
 * LOG A BIG BROTHER STATUS MESSAGE
 * THE MESSAGE GOES INTO THE DIRECTORY SPECIFIED BY BBLOG
 * MESSAGE FORMAT IS:
 * 	machine.area status message
 *
 * THE MESSAGE IS PLACED IN A FILE NAMED machine.area
 */

int bblog(msg)
char *msg;
{
        FILE *bbfile;
        int len;
        char fname[MAXLINE];				/* FILENAME */
	char filename[MAXLINE];
        char *name;

        if (bblogdir) {
                sscanf(msg, "%s", filename);

                len = strlen(filename);       
                if (strlen(bblogdir) + len + 2 > MAXLINE)
                {
                        /* too large for allocated memory, skip it */
			debug("FILENAME TOO BIG - IGNORED\n");
                        return;
                }

                /* 
		 * THANKS TO Jac Kersing <J.Kersing@the-box.com>
		 * SMM: IGNORE ATTEMPTS TO WRITE TO PARENT DIRECTORIES
		 * AND CHECK FOR BUFFER OVERFLOWS
                 */
                name = (char *)strrchr(filename,'/');
                if (name != (char *)NULL) {		/* FOUND A FUNNY */
			debug("ILLEGAL FILE NAME - IGNORED\n");
			return;				/* GO AWAY */
		}

        	sprintf(fname, "%s/%s", bblogdir, filename);

        	debug("OPENING BBFILE %s\n", fname);
		/*
		 * THANKS TO: Andrey Alekseyev <fetch@muffin.arcadia.spb.ru> 
		 * More checking...
		 */
        	bbfile = fopen(fname, "w+");            /* APPEND IT */
		if (bbfile) {
        		msg += len + 1;                 /* SKIP FIRST ARGS */
        		debug("MSG IS %s....\n", msg);
        		fprintf(bbfile, "%s\n", msg);   /* PUKE */
        		fclose(bbfile);                 /* CLOSE */
		}
		else {
			debug("FAILED TO OPEN STATUS FILE %s\n", bbfile);
		}
	}
	else die("BB LOGGING DIRECTORY UNDEFINED\n");
}

/*
 * PUT AN ERROR INTO THE LOGFILE AND stderr AND DIE
 */
int die(msg)
char *msg;
{
        static char message[MAXLINE];
        sprintf(message, "bbd: %s\n", msg);

        syslog(LOG_DAEMON | LOG_CRIT , message);
        fprintf(stderr, message);
        exit(1);
}

/*
 * SMM: v.1.06b CLEAN THIS UP...
 * THANKS TO: Andrey Alekseyev <fetch@muffin.arcadia.spb.ru>
 * READS IN THE DATABASE OF IP ADDRESSES AND NETMASKS
 * FROM etc/security
 */
getsecurity() {
	FILE *sec;
	char line[MAXBUF];
	long iaddr;
	int cnt;

	debug("INTO GETSECURITY\n");
	cnt = 0;

	sec = fopen(security, "r");
	if (sec) {
		while(fgets(line, MAXBUF, sec)) {
			debug("LINE IS: %s\n", line);

			iaddr = inet_addr(line);
			if ((iaddr != -1)&&(cnt < MAXBUF - 1)) {
				debug("ADDING LINE %d %s\n", cnt, line);
				securedb[cnt] = iaddr;
				cnt++;
			}
			else debug("DISCARDING: %s\n", line);
		}
		close(sec);		/* CLOSE THE FILE */
	}
}

checksecurity(ipnum) 
unsigned long ipnum;
{
	int cnt = 0;
	int retval = 1;			/* TRUE */
	unsigned long chk;

	while (chk = securedb[cnt]) {
		retval = 0;		/* FALSE */
		debug("CHECKSECURITY CHECKING %u VS %u ", ipnum, chk);
		if (((int)ipnum & (int)chk) == chk) {
			debug("--> PASSED\n");
			return(1);
		}
		else debug("--> FAILED\n");
		cnt++;
	}
	debug("CHECKSECUITY DEFAULT RETURNING %d\n", retval);
	return(retval);
}
