/*
 * bbnet.c
 * BIG BROTHER NETWORK TESTING PROGRAM
 * Sean MacGuire - The MacLawran Group Inc.
 * Version 1.06b
 * Mar 19, 1998
 *
 * This program is Copyright (c) 1997, 1998
 * The MacLawran Group Inc.
 * All Rights Reserved
 *
 * Format: bbnet ARG
 *
 * ARG:		http://blah.com/bobo			NORMAL HTTP TESTING
 *		http://blah.com/http://bobo.com		PROXY TESTING
 *		blah.com		TELNET TESTING (OR ANY OTHER PORT)
 *
 * bbnet understands real machine names and port numbers
 * i.e. blah.com:21 will open a connection to the FTP server
 *
 * Displays whatever answers on that port/address to stdout
 *
 * Returns: 	0	OK
 *		1	Host not found
 *		2	Can't connect to server
 *		3	Timeout
 */

#include "bb.h"		/* THE BIG BROTHER INCLUDE FILE */
#include <netdb.h>	/* FOR HOST INFO */

int http=0;		/* ARE WE HTTP TESTING? */
unsigned long ip;	/* THE IP ADDRESS WE'RE TESTING */
int port;		/* THE PORT NUMBER WE'RE TESTING */
char *dir;		/* DIRECTORY */

jmp_buf env;  

main(argc, argv)
int argc;
char *argv[];
{
	int p;
	char *machine, *ptr;
	if (argc == 3) p=80;		/* HTTP TEST */
	else if (argc == 2) p=23;	/* TELNET */
	else {
		fprintf(stderr, "bbnet: incorrect number of arguments\n");
		fprintf(stderr, "Format: bbnet machine[:port]|http-request\n");
		exit(1);
	}
	/*
	 * PARSE THE ARGUMENT.  THIS SHOULD BE A DROP-IN REPLACEMENT
	 * FOR lynx, UGH.  THAT MEANS THE FIRST ARGUMENT MAY BE A REALLY
	 * NASTY THING.
	 */
	machine = (char *)malloc(MAXLINE);
	strcpy(machine, argv[1]);

	if (strncmp(machine, "http://", 7) == 0) {
		http = 1;
		debug("We have an http request\n");
		machine += 7;
		port = 80;	/* STANDARD HTTP PORT */
	}
	else port = 23;		/* TELNET DEFAULT */
	setup(machine);		/* DO SETUP */
	bnetsend();		/* DO TEST */
	exit(0);
}

/*
 * THIS FUNCTION SETS UP EVERYTHING WE NEED TO CALL THE PORT
 */
setup(box)
char *box;
{
	char *eob = 0, *ptr,*newdir;
	struct hostent *ho = 0;

	ptr = (char *)strchr(box, ':');
	if (ptr != (char *)NULL) {
		debug("SCANNING FOR A PORT %s\n", ptr);
		/*
		 * SMM v1.06b
		 * FOR PROXY TESTING... THERE WILL BE ANOTHER
		 * COLON ':' BUT IT'LL BE FOR THE http: REQUEST
		 * SO CHECK THIS BY MAKING SURE THERE IS NO PORT #
		 */
		if (sscanf(ptr, ":%d", &port) != 0) eob = ptr;
	}
	debug("GOT PORT: %d\n", port);
	dir = (char *)strchr(box, '/');

	if (dir) {
		/*
		 * CHECK FOR A PROXY REQUEST
		 */
		if (strncmp(dir, "/http://", 8) == 0) {
			if (eob == 0) eob = dir;	
			dir++;		/* SKIP LEADING SLASH */
			debug("PROXY REQUEST for %s\n", dir);
		}
		newdir = (char *)malloc(strlen(dir) + 1);
		strcpy(newdir, dir);
		debug("DIRECTORY = %s\n", newdir);
		*dir = '\0';
	}
	/*
         * From: Doug White <dwhite@gdi.uoregon.edu>
	 */
	else  /* no dir was specified, default to / */
	{
		debug("NO DIR\n");
		newdir = (char *)malloc(2);
 		strcpy(newdir, "/");
	}

	if (eob) *eob = '\0';		/* NOW GET MACHINE */

	debug("BOX: %s\n", box);

	if((ip = inet_addr(box)) == -1) { /* IS IT IN DOT FMT */
		if (*box == '\0') {
			debug("BOX IS LOCAL\n");
			ho = gethostbyname("localhost");
		}
		else ho = gethostbyname(box);

		if (ho != (struct hostent *)NULL) {
			ip = (unsigned long)ho->h_addr;
		}
		else {	/* TEST FOR AN IP-ADDRESS */
			fprintf(stderr, "Invalid host request: %s\n", box);
			exit(1);	/* NO HOST */
		}
	/*
	 * IF WE MAKE IT HERE, WE HAVE TO COPY THE HOSTNAME 
	 * INTO THE ADDRESS STRUCTURE
	 */
		memcpy(&ip, ho->h_addr, ho->h_length);
	}
	dir=newdir;			/* SET DIRECTORY */
}

bnetsend()
{
 	int n, sockfd, newsockfd, child, len, timeout();
        struct sockaddr_in serv_addr, cli_addr;
	char line[MAXLINE];

        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = ip;
        serv_addr.sin_port = htons(port);

	/*
	 * THIS IS A LITTLE NASTY
	 * IF THE SERVER'S NOT UP, THIS WILL WAIT A LONG TIME BEFORE
	 * IT TIMES OUT.  SO HANDLE THAT HERE... IT GETS 30 SECONDS TO ANSWER
	 */
 	if (setjmp(env) != 0) {
                debug("CALLING SETJMP\n");
        }
        signal(SIGALRM, (void *)timeout);
        alarm(30);

        /* OPEN SOCKET */
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "\nbbnet: CAN'T OPEN SOCKET\n");
		exit(2);
        }
        if (connect(sockfd, (struct sockaddr *) &serv_addr,
                        sizeof(serv_addr)) < 0) {
                fprintf(stderr, "\nbbnet: CAN'T CONNECT TO SERVER\n");
		/*
		 * THANKS TO: Douwe Dijkstra <D.Dijkstra@Twinfo.nl>
		 */ 
		exit(2);
        }
	/*
	 * HTTP REQUESTS ARE SPECIAL
	 * PROPER HEAD PROTOCOL NOW INCLUDED...
	 * Thanks: Don Sullivan (dsullivan@gaia.arc.nasa.gov)
	 * EXTRA \r FOR Brian Lenihan <brianl@real.com> AND ROXEN
	 */
	if (http) {
		if (dir) sprintf(line, "HEAD %s HTTP/1.0\r\n\r\n", dir); 
		else sprintf(line, "HEAD / HTTP/1.0\r\n\r\n");
	}
	/*
	 * Thanks to Jon Lewis <jlewis@inorganic5.fdt.net>
	 */
	else sprintf(line, "quit\r\n");

        n = send(sockfd, line, strlen(line),0);
	line[0]='\0';
	n = recv(sockfd, line, MAXLINE, 0);
	if (n > 0) line[n]='\0';	/* Shun-Jee Liu <liu@hermes.com.tw> */
	/*
	 * SMM - 26 DEC 97
	 * SOME TESTING SHOULDN'T BE DISPLAYED 'CAUSE IT'S GROSS
	 * THANKS TO: Per.E.Berger@telia.se
	 */
	if (port == 23) {
		puts("Telnet test OK");
	}
	else puts(line);			/* DUMP OUT THE LINE */
	close(sockfd);
	return(n);
}

timeout()
{
	fprintf(stderr, "bbnet: CONNECTION TIMED OUT...\n");
	exit(3);
}


